"""Finam Account MCP - действия связанные с получением данных об аккаунте"""

from fastmcp import FastMCP
from finam_trade_api.account import GetTransactionsResponse, GetTradesResponse

from src.servers.utils import get_finam_client
from src.tradeapi.models import GetAccountResponse, Timestamp

account_mcp = FastMCP(name="FinamAccountServer")


@account_mcp.tool(tags={"account"})
async def get_info() -> GetAccountResponse:
    """Получение информации по конкретному счету (статус и тип аккаунта, доступные средства, дневная прибыль, открытые позиции (количество, средняя цена, прибыль/убыток), тип портфеля)"""
    return await get_finam_client().get_account_info()


@account_mcp.tool(tags={"account"})
async def get_transactions(start_time: Timestamp, end_time: Timestamp, limit: int = 10) -> GetTransactionsResponse:
    """Получение списка транзакций аккаунта"""
    return await get_finam_client().get_transactions(start_time, end_time, limit)


@account_mcp.tool(tags={"account"})
async def get_trades(start_time: Timestamp, end_time: Timestamp, limit: int = 10) -> GetTradesResponse:
    """Получение истории по сделкам аккаунта"""
    return await get_finam_client().get_trades(start_time, end_time, limit)



from datetime import datetime
from decimal import Decimal
from enum import Enum

from pydantic import BaseModel, Field

from src.tradeapi.models import Symbol


class Side(str, Enum):
    """Сторона сделки"""

    BUY = "SIDE_BUY"
    SELL = "SIDE_SELL"


class OrderType(str, Enum):
    """Тип заявки"""

    UNSPECIFIED = "ORDER_TYPE_UNSPECIFIED"
    MARKET = "ORDER_TYPE_MARKET"
    LIMIT = "ORDER_TYPE_LIMIT"
    STOP = "ORDER_TYPE_STOP"
    STOP_LIMIT = "ORDER_TYPE_STOP_LIMIT"
    MULTI_LEG = "ORDER_TYPE_MULTI_LEG"


class TimeInForce(str, Enum):
    """Срок действия заявки"""

    UNSPECIFIED = "TIME_IN_FORCE_UNSPECIFIED"
    DAY = "TIME_IN_FORCE_DAY"
    GOOD_TILL_CANCEL = "TIME_IN_FORCE_GOOD_TILL_CANCEL"
    GOOD_TILL_CROSSING = "TIME_IN_FORCE_GOOD_TILL_CROSSING"
    EXT = "TIME_IN_FORCE_EXT"
    ON_OPEN = "TIME_IN_FORCE_ON_OPEN"
    ON_CLOSE = "TIME_IN_FORCE_ON_CLOSE"
    IOC = "TIME_IN_FORCE_IOC"
    FOK = "TIME_IN_FORCE_FOK"


class StopCondition(str, Enum):
    """Условие срабатывания стоп заявки"""

    UNSPECIFIED = "STOP_CONDITION_UNSPECIFIED"
    LAST_UP = "STOP_CONDITION_LAST_UP"
    LAST_DOWN = "STOP_CONDITION_LAST_DOWN"


class ValidBefore(str, Enum):
    """Срок действия условной заявки"""

    UNSPECIFIED = "VALID_BEFORE_UNSPECIFIED"
    END_OF_DAY = "VALID_BEFORE_END_OF_DAY"
    GOOD_TILL_CANCEL = "VALID_BEFORE_GOOD_TILL_CANCEL"
    GOOD_TILL_DATE = "VALID_BEFORE_GOOD_TILL_DATE"


class OrderStatus(str, Enum):
    """Статус заявки"""

    UNSPECIFIED = "ORDER_STATUS_UNSPECIFIED"
    NEW = "ORDER_STATUS_NEW"
    PARTIALLY_FILLED = "ORDER_STATUS_PARTIALLY_FILLED"
    FILLED = "ORDER_STATUS_FILLED"
    DONE_FOR_DAY = "ORDER_STATUS_DONE_FOR_DAY"
    CANCELED = "ORDER_STATUS_CANCELED"
    REPLACED = "ORDER_STATUS_REPLACED"
    PENDING_CANCEL = "ORDER_STATUS_PENDING_CANCEL"
    REJECTED = "ORDER_STATUS_REJECTED"
    SUSPENDED = "ORDER_STATUS_SUSPENDED"
    PENDING_NEW = "ORDER_STATUS_PENDING_NEW"
    EXPIRED = "ORDER_STATUS_EXPIRED"
    FAILED = "ORDER_STATUS_FAILED"
    FORWARDING = "ORDER_STATUS_FORWARDING"
    WAIT = "ORDER_STATUS_WAIT"
    DENIED_BY_BROKER = "ORDER_STATUS_DENIED_BY_BROKER"
    REJECTED_BY_EXCHANGE = "ORDER_STATUS_REJECTED_BY_EXCHANGE"
    WATCHING = "ORDER_STATUS_WATCHING"
    EXECUTED = "ORDER_STATUS_EXECUTED"
    DISABLED = "ORDER_STATUS_DISABLED"
    LINK_WAIT = "ORDER_STATUS_LINK_WAIT"
    SL_GUARD_TIME = "ORDER_STATUS_SL_GUARD_TIME"
    SL_EXECUTED = "ORDER_STATUS_SL_EXECUTED"
    SL_FORWARDING = "ORDER_STATUS_SL_FORWARDING"
    TP_GUARD_TIME = "ORDER_STATUS_TP_GUARD_TIME"
    TP_EXECUTED = "ORDER_STATUS_TP_EXECUTED"
    TP_CORRECTION = "ORDER_STATUS_TP_CORRECTION"
    TP_FORWARDING = "ORDER_STATUS_TP_FORWARDING"
    TP_CORR_GUARD_TIME = "ORDER_STATUS_TP_CORR_GUARD_TIME"


class Leg(BaseModel):
    """Лег для мульти лег заявки"""

    symbol: Symbol
    quantity: Decimal = Field(..., description="Количество")
    side: Side = Field(..., description="Сторона")


class Order(BaseModel):
    """Информация о заявке"""

    symbol: Symbol
    quantity: Decimal = Field(description="Количество в шт.")
    side: Side = Field(description="Сторона (long или short)")
    type: OrderType = Field(description="Тип заявки")
    time_in_force: TimeInForce = Field(
        TimeInForce.UNSPECIFIED, description="Срок действия заявки"
    )
    limit_price: Decimal | None = Field(
        None, description="Необходимо для лимитной и стоп лимитной заявки"
    )
    stop_price: Decimal | None = Field(
        None, description="Необходимо для стоп рыночной и стоп лимитной заявки"
    )
    stop_condition: StopCondition = Field(
        StopCondition.UNSPECIFIED,
        description="Необходимо для стоп рыночной и стоп лимитной заявки",
    )
    legs: list[Leg] | None = Field(None, description="Необходимо для мульти лег заявки")
    client_order_id: str | None = Field(
        None, max_length=20, description="Уникальный идентификатор заявки"
    )
    valid_before: ValidBefore | None = Field(
        None, description="Срок действия условной заявки"
    )
    comment: str | None = Field(None, max_length=128, description="Метка заявки")


class OrderState(BaseModel):
    """Состояние заявки"""

    order_id: str = Field(..., description="Идентификатор заявки")
    exec_id: str | None = Field(None, description="Идентификатор исполнения")
    status: OrderStatus = Field(..., description="Статус заявки")
    order: Order = Field(..., description="Заявка")
    transact_at: datetime | None = Field(
        None, description="Дата и время выставления заявки"
    )
    accept_at: datetime | None = Field(None, description="Дата и время принятия заявки")
    withdraw_at: datetime | None = Field(None, description="Дата и время отмены заявки")


class OrdersResponse(BaseModel):
    """Список торговых заявок"""

    orders: list[OrderState] = Field(..., description="Заявки")



import pytest
from fastmcp import Client

from src.main import finam_mcp


@pytest.fixture(scope="function")
async def mcp_client():
    """
    Создание in-memory MCP клиента для e2e тестирования
    """
    client = Client(finam_mcp)

    async with client:
        yield client


TEST_STOCK_SYMBOLS_RU = ["YDEX@MISX", "SBER@MISX"]
TEST_STOCK_SYMBOLS_US = ["AAPL@XNGS", "SPY@ARCX", "KO@XNYS"]

TEST_STOCK_SYMBOLS = TEST_STOCK_SYMBOLS_US
TEST_INVALID_SYMBOL = "INVALID@SYMBOL"



from fastmcp import FastMCP
from fastmcp.server.middleware.error_handling import ErrorHandlingMiddleware

from src.config import settings
from src.middleware import FinamCredentialsMiddleware
from src.servers.account import account_mcp
from src.servers.assets import assets_mcp
from src.servers.market_data import market_data_mcp
from src.servers.order import order_mcp

finam_mcp = FastMCP("FinamMCP", include_tags=settings.INCLUDE_SERVERS)
finam_mcp.mount(account_mcp, prefix="account")
finam_mcp.mount(market_data_mcp, prefix="market_data")
finam_mcp.mount(assets_mcp, prefix="assets")
finam_mcp.mount(order_mcp, prefix="order")

finam_mcp.add_middleware(FinamCredentialsMiddleware())
finam_mcp.add_middleware(ErrorHandlingMiddleware())

if __name__ == "__main__":
    finam_mcp.run(transport="http", host="127.0.0.1", port=3000)



from typing import Annotated

from finam_trade_api.account import GetTransactionsRequest, Position
from finam_trade_api.assets.model import Status
from finam_trade_api.base_client import FinamDecimal, FinamMoney
from pydantic import BaseModel, Field, AwareDatetime

Symbol: type[str] = Annotated[
    str,
    Field(
        description="Symbol in format: SYMBOL@MIC",
        pattern=r"^[A-Z0-9]+@[A-Z]+$",
        examples=["YDEX@MISX", "SBER@TQBR"],
    ),
]

Timestamp: type[AwareDatetime] = Annotated[
    AwareDatetime,
    Field(
        description="Timestamp in RFC 3339 format (ISO 8601 with timezone)",
        examples=["2024-01-15T10:30:00Z", "2024-01-15T10:30:00+03:00"],
    ),
]


class GetTradesRequest(GetTransactionsRequest):
    account_id: str
    start_time: str
    end_time: str
    limit: int


class AssetParamsResponse(BaseModel):
    symbol: Symbol
    account_id: str
    tradeable: bool
    longable: Status | None = None
    shortable: Status | None = None
    long_risk_rate: FinamDecimal | None = None
    long_collateral: FinamMoney | None = None
    short_risk_rate: FinamDecimal | None = None
    short_collateral: FinamMoney | None = None


class GetAccountResponse(BaseModel):
    account_id: str
    type: str
    status: str
    positions: list[Position] = Field(default_factory=list)
    cash: list[FinamMoney] = Field(default_factory=list)
    open_account_date: Timestamp
    first_non_trade_date: Timestamp
    equity: FinamDecimal | None = None
    unrealized_profit: FinamDecimal | None = None



from enum import Enum
from typing import Any

import httpx
from finam_trade_api import TokenManager


class RequestMethod(str, Enum):
    """
    Перечисление методов HTTP-запросов.
    """

    POST = "post"
    PUT = "put"
    GET = "get"
    DELETE = "delete"


class HttpxClient:
    """
    Базовый клиент для выполнения HTTP-запросов с использованием токенов аутентификации.

    Атрибуты:
        _token_manager (TokenManager): Менеджер токенов для управления JWT-токеном.
        _base_url (str): Базовый URL для всех запросов.
    """

    def __init__(
        self, token_manager: TokenManager, url: str = "https://api.finam.ru/v1"
    ):
        self._token_manager = token_manager
        self._base_url = url

    @property
    def _auth_headers(self):
        return (
            {"Authorization": self._token_manager.jwt_token}
            if self._token_manager.jwt_token
            else None
        )

    async def _exec_request(
        self, method: str, url: str, payload=None, **kwargs
    ) -> tuple[Any, bool]:
        uri = f"{self._base_url}{url}"

        async with httpx.AsyncClient(
            http2=True, timeout=20, headers=self._auth_headers
        ) as client:
            response = await client.request(method, uri, json=payload, **kwargs)

            if response.status_code != 200:
                if "application/json" not in response.headers.get("content-type", ""):
                    response.raise_for_status()
                return response.json(), False
            return response.json(), True



from datetime import datetime, timedelta, timezone
import pytest
import re
from finam_trade_api.instruments import (
    TimeFrame,
    BarsResponse,
    QuoteResponse,
    OrderBookResponse,
    TradesResponse,
)

from tests.conftest import TEST_INVALID_SYMBOL, TEST_STOCK_SYMBOLS


@pytest.mark.parametrize("symbol", TEST_STOCK_SYMBOLS)
async def test_get_bars(mcp_client, symbol):
    end_time = datetime.now(timezone.utc).replace(
        hour=0, minute=0, second=0, microsecond=0
    )
    start_time = end_time - timedelta(days=7)

    response = await mcp_client.call_tool(
        "market_data_get_bars",
        arguments={
            "symbol": symbol,
            "start_time": start_time,
            "end_time": end_time,
            "timeframe": TimeFrame.TIME_FRAME_D,
        },
    )

    assert response.is_error is False
    assert BarsResponse.model_validate(response.structured_content)


async def test_get_bars_without_isoformat(mcp_client):
    end_time = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)
    start_time = end_time - timedelta(days=7)

    response = await mcp_client.call_tool(
        "market_data_get_bars",
        arguments={
            "symbol": "SBER@MISX",
            "start_time": start_time,
            "end_time": end_time,
            "timeframe": TimeFrame.TIME_FRAME_D,
        },
        raise_on_error=False,
    )

    assert response.is_error is True

    error_text = re.sub(r"\s+", " ", response.content[0].text).strip()
    assert "start_time Input should have timezone info" in error_text
    assert "end_time Input should have timezone info" in error_text


@pytest.mark.parametrize("symbol", TEST_STOCK_SYMBOLS)
async def test_get_last_quote(mcp_client, symbol):
    response = await mcp_client.call_tool(
        "market_data_get_last_quote", arguments={"symbol": symbol}
    )

    assert response.is_error is False
    assert QuoteResponse.model_validate(response.structured_content)


@pytest.mark.parametrize("symbol", TEST_STOCK_SYMBOLS)
async def test_get_last_trades(mcp_client, symbol):
    response = await mcp_client.call_tool(
        "market_data_get_last_trades",
        arguments={"symbol": symbol},
        raise_on_error=False,
    )

    if response.is_error is False:
        assert TradesResponse.model_validate(response.structured_content)
    else:
        # Trades data can be not found
        error_text = response.content[0].text
        assert "Trades not found" in error_text


@pytest.mark.parametrize("symbol", TEST_STOCK_SYMBOLS)
async def test_get_order_book(mcp_client, symbol):
    response = await mcp_client.call_tool(
        "market_data_get_order_book", arguments={"symbol": symbol}
    )

    assert response.is_error is False
    assert OrderBookResponse.model_validate(response.structured_content)


@pytest.mark.parametrize(
    "tool",
    [
        "market_data_get_order_book",
        "market_data_get_last_trades",
        "market_data_get_last_quote",
        "market_data_get_bars",
    ],
)
async def test_get_order_book_invalid_symbol(mcp_client, tool):
    """Тест обработки ошибки при неправильном symbol"""
    params = {"symbol": TEST_INVALID_SYMBOL}
    if tool == "market_data_get_bars":
        end_time = datetime.now(timezone.utc).replace(
            hour=0, minute=0, second=0, microsecond=0
        )
        start_time = end_time - timedelta(days=7)

        params["timeframe"] = TimeFrame.TIME_FRAME_D
        params["start_time"] = start_time.isoformat()
        params["end_time"] = end_time.isoformat()

    response = await mcp_client.call_tool(tool, arguments=params, raise_on_error=False)

    assert response.is_error is True
    assert response.structured_content is None
    error_text = response.content[0].text
    assert "Security id doesn't exist" in error_text



import pytest
from finam_trade_api.assets import (
    OptionsChainResponse,
    AssetsResponse,
    ExchangesResponse,
)

from conftest import TEST_STOCK_SYMBOLS


async def test_get_assets(mcp_client):
    response = await mcp_client.call_tool("assets_get_list")
    assert response.is_error is False
    assets = AssetsResponse.model_validate(response.structured_content)
    assert assets.assets


async def test_get_exchanges(mcp_client):
    response = await mcp_client.call_tool("assets_get_exchanges")
    assert response.is_error is False
    exchanges = ExchangesResponse.model_validate(response.structured_content)
    assert exchanges.exchanges


@pytest.mark.parametrize("symbol", TEST_STOCK_SYMBOLS)
async def test_get_options_chain(mcp_client, symbol):
    response = await mcp_client.call_tool(
        "assets_get_options_chain", arguments={"symbol": symbol}
    )

    assert response.is_error is False
    assert OptionsChainResponse.model_validate(response.structured_content)



from datetime import datetime

from finam_trade_api import Client, TokenManager, ErrorModel
from finam_trade_api.account import GetTransactionsRequest
from finam_trade_api.assets import AssetsResponse
from finam_trade_api.instruments import (
    TimeFrame,
    TradesResponse,
    OrderBookResponse,
    QuoteResponse,
    BarsResponse,
)
from mcp.server.fastmcp.exceptions import ToolError

from src.tradeapi.base import HttpxClient, RequestMethod
from src.tradeapi.models import (
    GetTradesRequest,
    AssetParamsResponse,
    GetAccountResponse,
)
from src.tradeapi.order.orders import OrderClient


class FinamClient:
    def __init__(self, api_key, account_id):
        self.token_manager = TokenManager(api_key)
        self.client = Client(self.token_manager)
        self.client.orders = OrderClient(self.token_manager)  # доделка
        self.httpx_client = HttpxClient(self.token_manager)  # доделка
        self.account_id = account_id

    @classmethod
    async def create(cls, api_key, account_id):
        instance = cls(api_key, account_id)
        token = await instance._exec_request(
            RequestMethod.POST,
            "/sessions",
            payload={"secret": instance.token_manager.token},
        )
        instance.token_manager.set_jwt_token(token["token"])
        return instance

    """ Helper """

    async def _exec_request(self, method: RequestMethod, url: str, **kwargs) -> dict:
        response, ok = await self.httpx_client._exec_request(method, url, **kwargs)

        if not ok:
            err = ErrorModel(**response)
            raise ToolError(
                f"code={err.code} | message={err.message} | details={err.details}"
            )
        return response

    """ Аккаунт """

    async def get_account_info(self):
        account_client = self.client.account
        return GetAccountResponse(
            **await self._exec_request(
                RequestMethod.GET, f"{account_client._url}/{self.account_id}"
            )
        )

    async def get_transactions(
        self, start_time: datetime, end_time: datetime, limit: int = 10
    ):
        return await self.client.account.get_transactions(
            GetTransactionsRequest(
                account_id=self.account_id,
                start_time=start_time.isoformat(),
                end_time=end_time.isoformat(),
                limit=limit,
            )
        )

    async def get_trades(
        self, start_time: datetime, end_time: datetime, limit: int = 10
    ):
        return await self.client.account.get_trades(
            GetTradesRequest(
                account_id=self.account_id,
                start_time=start_time.isoformat(),
                end_time=end_time.isoformat(),
                limit=limit,
            )
        )

    """ Assets """

    async def get_assets(self):
        return AssetsResponse(**await self._exec_request(RequestMethod.GET, "/assets"))

    async def get_asset(self, symbol: str):
        return await self.client.assets.get_asset(symbol, self.account_id)

    async def get_asset_params(self, symbol: str):
        return AssetParamsResponse(
            **await self._exec_request(
                RequestMethod.GET,
                f"/assets/{symbol}/params",
                params={"account_id": self.account_id},
            )
        )

    async def get_exchanges(self):
        return await self.client.assets.get_exchanges()

    async def get_options_chain(self, underlying_symbol: str):
        return await self.client.assets.get_options_chain(underlying_symbol)

    async def get_schedule(self, symbol: str):
        return await self.client.assets.get_schedule(symbol)

    """ Market Data """

    async def get_bars(
        self,
        symbol: str,
        start_time: datetime,
        end_time: datetime,
        timeframe: TimeFrame,
    ):
        return BarsResponse(
            **await self._exec_request(
                RequestMethod.GET,
                f"/instruments/{symbol}/bars",
                params={
                    "timeframe": timeframe.value,
                    "interval.start_time": start_time.isoformat(),
                    "interval.end_time": end_time.isoformat(),
                },
            )
        )

    async def get_last_quote(self, symbol: str):
        return QuoteResponse(
            **await self._exec_request(
                RequestMethod.GET,
                f"/instruments/{symbol}/quotes/latest",
            )
        )

    async def get_last_trades(self, symbol: str):
        return TradesResponse(
            **await self._exec_request(
                RequestMethod.GET,
                f"/instruments/{symbol}/trades/latest",
            )
        )

    async def get_order_book(self, symbol: str):
        return OrderBookResponse(
            **await self._exec_request(
                RequestMethod.GET,
                f"/instruments/{symbol}/orderbook",
            )
        )

    """ Orders """

    async def get_orders(self):
        """Получение списка заявок для аккаунта"""
        return await self.client.orders.get_orders(self.account_id)

    async def get_order(self, order_id: str):
        """Получение информации о конкретном ордере"""
        return await self.client.orders.get_order(order_id, self.account_id)

    async def place_order(self, order):
        """Выставление биржевой заявки"""
        return await self.client.orders.place_order(order, self.account_id)

    async def cancel_order(self, order_id: str):
        """Отмена биржевой заявки"""
        return await self.client.orders.cancel_order(order_id, self.account_id)



from fastmcp import FastMCP
from finam_trade_api.assets import (
    AssetsResponse,
    AssetResponse,
    ExchangesResponse,
    AssetParamsResponse,
    ScheduleResponse,
    OptionsChainResponse,
)

from src.servers.utils import get_finam_client
from src.tradeapi.models import Symbol

assets_mcp = FastMCP(name="FinamAssetsServer")


@assets_mcp.tool(tags={"assets"})
async def get(symbol: Symbol) -> AssetResponse:
    """Получение информации по конкретному инструменту (лот, шаг цены, дата экспирации фьючерса)"""
    return await get_finam_client().get_asset(symbol)


@assets_mcp.tool(tags={"assets"})
async def get_params(symbol: Symbol) -> AssetParamsResponse:
    """Получение торговых параметров по инструменту (операции лонг/шорт, гарантийное обеспечение, ставки риска)"""
    return await get_finam_client().get_asset_params(symbol)


@assets_mcp.tool(tags={"assets"})
async def get_list() -> AssetsResponse:
    """Получение списка доступных инструментов, их описание (символы, наименование)"""
    return await get_finam_client().get_assets()


@assets_mcp.tool(tags={"assets"})
async def get_exchanges() -> ExchangesResponse:
    """Получение списка доступных бирж, включая их названия и MIC-коды"""
    return await get_finam_client().get_exchanges()


@assets_mcp.tool(tags={"assets"})
async def get_options_chain(symbol: Symbol) -> OptionsChainResponse:
    """Получение цепочки опционов для базового актива"""
    return await get_finam_client().get_options_chain(symbol)


@assets_mcp.tool(tags={"assets"})
async def get_schedule(symbol: Symbol) -> ScheduleResponse:
    """Получение расписания торгов для указанного инструмента"""
    return await get_finam_client().get_schedule(symbol)



"""CLI entry point for finam-mcp server."""

import sys
import os

import click


@click.command()
@click.option(
    "--transport",
    type=click.Choice(["stdio", "http"]),
    default="stdio",
    help="Transport method: stdio (default) or http",
)
@click.option(
    "--host",
    default="127.0.0.1",
    help="Host to bind for http transport (default: 127.0.0.1)",
)
@click.option(
    "--port",
    type=int,
    default=3000,
    help="Port to bind for http transport (default: 3000)",
)
@click.version_option(version="0.1.0", prog_name="finam-mcp")
def main(transport: str, host: str, port: int) -> None:
    """
    Finam MCP Server - Finam Trade API integration for Model Context Protocol.

    Запускает Finam MCP сервер для интеграции с Finam Trade API.
    Креденшалы берутся из переменных окружения:
    - FINAM_API_KEY (обязательно)
    - FINAM_ACCOUNT_ID (обязательно)
    - INCLUDE_SERVERS (опционально, например: account,market_data)

    Examples:
        finam-mcp                           # Start with stdio transport
        finam-mcp --transport http          # Start HTTP server on default port
        finam-mcp --transport http --port 8000  # Custom port
    """
    # Проверяем наличие обязательных переменных окружения
    api_key = os.getenv("FINAM_API_KEY")
    account_id = os.getenv("FINAM_ACCOUNT_ID")

    missing_vars = []
    if not api_key:
        missing_vars.append("FINAM_API_KEY")
    if not account_id:
        missing_vars.append("FINAM_ACCOUNT_ID")

    if missing_vars:
        click.echo("Error: Required environment variables are not set:", err=True)
        for var in missing_vars:
            click.echo(f"  - {var}", err=True)
        click.echo(
            "\nPlease set the required environment variables and try again.", err=True
        )
        click.echo("Example:", err=True)
        click.echo("  export FINAM_API_KEY=your_api_key", err=True)
        click.echo("  export FINAM_ACCOUNT_ID=your_account_id", err=True)
        sys.exit(1)

    from src.main import finam_mcp

    # Показываем информацию о включённых серверах
    include_servers = finam_mcp.include_tags
    if include_servers:
        click.echo(
            f"Starting Finam MCP server with enabled modules: {', '.join(include_servers)}",
            err=True,
        )
    else:
        click.echo(
            "Starting Finam MCP server with all modules enabled (account, assets, market_data, order)",
            err=True,
        )

    # Показываем информацию о транспорте
    if transport == "http":
        click.echo(f"Transport: HTTP at http://{host}:{port}", err=True)
    else:
        click.echo("Transport: STDIO", err=True)

    click.echo("", err=True)  # Пустая строка для разделения

    try:
        # Запускаем сервер с выбранным транспортом
        if transport == "stdio":
            finam_mcp.run()
        else:
            finam_mcp.run(transport="http", host=host, port=port)
    except KeyboardInterrupt:
        click.echo("\nServer stopped by user", err=True)
        sys.exit(0)
    except Exception as e:
        click.echo(f"Error starting server: {e}", err=True)
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Finam Market MCP - действия связанные с получением рыночных данных"""

from fastmcp import FastMCP
from finam_trade_api.instruments import (
    BarsResponse,
    QuoteResponse,
    OrderBookResponse,
    TimeFrame,
    TradesResponse,
)

from src.servers.utils import get_finam_client
from src.tradeapi.models import Symbol, Timestamp

market_data_mcp = FastMCP(name="FinamMarketDataServer")


@market_data_mcp.tool(tags={"market_data"})
async def get_bars(
    symbol: Symbol, start_time: Timestamp, end_time: Timestamp, timeframe: TimeFrame
) -> BarsResponse:
    """Получение исторических данных по инструменту (агрегированные свечи)"""
    return await get_finam_client().get_bars(symbol, start_time, end_time, timeframe)


@market_data_mcp.tool(tags={"market_data"})
async def get_last_quote(symbol: Symbol) -> QuoteResponse:
    """получение последней котировки инструмента (цена покупки/продажи, цена открытия/закрытия, цена последней сделки, дневной объем сделок, объем покупки/продажи)"""
    return await get_finam_client().get_last_quote(symbol)


@market_data_mcp.tool(tags={"market_data"})
async def get_last_trades(symbol: Symbol) -> TradesResponse:
    """Получение списка последних сделок по инструменту"""
    return await get_finam_client().get_last_trades(symbol)


@market_data_mcp.tool(tags={"market_data"})
async def get_order_book(symbol: Symbol) -> OrderBookResponse:
    """Получение текущего стакана по инструменту"""
    return await get_finam_client().get_order_book(symbol)



from fastmcp.exceptions import ToolError
from fastmcp.server.dependencies import get_http_headers
from fastmcp.server.middleware import Middleware, MiddlewareContext

from src.config import settings
from src.tradeapi.finam_client import FinamClient


class FinamCredentialsMiddleware(Middleware):
    """Middleware для создания FinamClient из заголовков и добавления в контекст."""

    async def on_call_tool(self, context: MiddlewareContext, call_next):
        """Перехватываем все вызовы tools."""

        # Получаем заголовки из HTTP запроса
        headers = get_http_headers()

        # Извлекаем необходимые заголовки
        api_key = headers.get("finam-api-key") or settings.FINAM_API_KEY
        account_id = headers.get("finam-account-id") or settings.FINAM_ACCOUNT_ID

        # Проверяем наличие обязательных заголовков
        if not api_key or not account_id:
            raise ToolError(
                "Missing required headers/env variables: FINAM-API-KEY and FINAM-ACCOUNT-ID are required"
            )

        # Создаем клиент Finam
        finam_client = await FinamClient.create(api_key=api_key, account_id=account_id)

        # Сохраняем клиента в state контекста
        if context.fastmcp_context:
            context.fastmcp_context.set_state("finam_client", finam_client)

        # Продолжаем выполнение
        return await call_next(context)



"""Finam Trade MCP - действия связанные с торговлей"""

from fastmcp import FastMCP

from src.servers.utils import get_finam_client
from src.tradeapi.order.models import OrdersResponse, OrderState, Order

order_mcp = FastMCP(name="FinamOrderServer")


@order_mcp.tool(tags={"order"})
async def get_list() -> OrdersResponse:
    """Получение списка заявок для аккаунта"""
    return await get_finam_client().get_orders()


@order_mcp.tool(tags={"order"})
async def get(order_id: str) -> OrderState:
    """Получение информации о конкретном ордере"""
    return await get_finam_client().get_order(order_id)


@order_mcp.tool(tags={"order"}, meta={"sensitive": True})
async def place(order: Order) -> OrderState:
    """Выставление биржевой заявки"""
    return await get_finam_client().place_order(order)


@order_mcp.tool(tags={"order"}, meta={"sensitive": True})
async def cancel(order_id: str) -> OrderState:
    """Отмена биржевой заявки"""
    return await get_finam_client().cancel_order(order_id)



from decimal import Decimal

from finam_trade_api import TokenManager, FinamTradeApiError, ErrorModel
from finam_trade_api.base_client import BaseClient

from src.tradeapi.order.models import OrdersResponse, OrderState, Order


class OrderClient(BaseClient):
    def __init__(self, token_manager: TokenManager):
        super().__init__(token_manager)
        self._url = "/accounts/{account_id}/orders"

    async def get_orders(self, account_id: str) -> OrdersResponse:
        """Получение списка заявок для аккаунта"""
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._url.format(account_id=account_id),
        )

        if not ok:
            err = ErrorModel(**response)
            raise FinamTradeApiError(
                f"code={err.code} | message={err.message} | details={err.details}"
            )

        return OrdersResponse(**response)

    async def get_order(self, order_id: str, account_id: str) -> OrderState:
        """Получение информации о конкретном ордере"""
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._url.format(account_id=account_id) + f"/{order_id}",
        )

        if not ok:
            err = ErrorModel(**response)
            raise FinamTradeApiError(
                f"code={err.code} | message={err.message} | details={err.details}"
            )

        return OrderState(**response)

    async def place_order(self, order: Order, account_id: str) -> OrderState:
        """Выставление биржевой заявки"""
        order_body = {
            key: ({"value": str(value)} if isinstance(value, Decimal) else value)
            for key, value in order.model_dump(exclude_unset=True).items()
        }
        response, ok = await self._exec_request(
            self.RequestMethod.POST,
            self._url.format(account_id=account_id),
            payload=order_body,
        )

        if not ok:
            err = ErrorModel(**response)
            raise FinamTradeApiError(
                f"code={err.code} | message={err.message} | details={err.details}"
            )

        return OrderState(**response)

    async def cancel_order(self, order_id: str, account_id: str) -> OrderState:
        """Отмена биржевой заявки"""
        response, ok = await self._exec_request(
            self.RequestMethod.DELETE,
            self._url.format(account_id=account_id) + f"/{order_id}",
        )

        if not ok:
            err = ErrorModel(**response)
            raise FinamTradeApiError(
                f"code={err.code} | message={err.message} | details={err.details}"
            )

        return OrderState(**response)



from pathlib import Path

from pydantic import field_validator
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    PROJECT_PATH: Path = Path(__file__).parent.parent

    FINAM_API_KEY: str | None = None
    FINAM_ACCOUNT_ID: str | None = None
    INCLUDE_SERVERS: list[str] | None = None

    @field_validator("INCLUDE_SERVERS", mode="before")
    @classmethod
    def set_include_server_tags(cls, v: str | list[str]) -> list[str] | None:
        if v:
            if isinstance(v, str) and not v.startswith("[") and v:
                return [i.strip() for i in v.split(",")]
            elif isinstance(v, list | str):
                return v
        return None

    model_config = SettingsConfigDict(
        case_sensitive=True,
        env_file=PROJECT_PATH / ".env",
        env_file_encoding="utf-8",
        extra="ignore",
    )


settings = Settings()
